import * as rslConstants from './smrtfill.constants.js';
import { exceptionalError } from './smrtfill.js';
import { logger } from './smrtfill.js';
import util from 'util';
export function valIsEmpty(val) {
    return (!val || val === "");
}
// Validate inputs before trying to create a new shipment
export function validateNewShipment(orderNumber, orderItems, customerDetails, niceShipMethod) {
    let errMsgs = [];
    if (valIsEmpty(orderNumber)) {
        errMsgs.push("Order Number Is Required");
    }
    const mandatoryRxDeets = [
        "name", "addressOne", "city", "state", "country", "zip"
    ];
    if (valIsEmpty(niceShipMethod)) {
        errMsgs.push("Shipping method is required");
    }
    if (!rslConstants.standardShippingMethods.includes(niceShipMethod)) {
        errMsgs.push("Invalid shipping method selected");
    }
    for (const mandatoryKey of mandatoryRxDeets) {
        //@ts-ignore
        if (!customerDetails[mandatoryKey] || customerDetails[mandatoryKey] === "") {
            errMsgs.push(`Missing mandatory customer address field ${mandatoryKey}`);
        }
    }
    if (Object.keys(orderItems).length === 0) {
        errMsgs.push(`At least one SKU:QTY entry is required`);
    }
    for (const [sku, skuDeets] of Object.entries(orderItems)) {
        if (valIsEmpty(sku)) {
            errMsgs.push(`Empty or null skus found in shipment item list`);
        }
        else {
            if (typeof (skuDeets.qty) !== 'number') {
                errMsgs.push(`Non-numeric qty found for sku ${sku}`);
            }
        }
    }
    if (errMsgs.length > 0) {
        logger.info(util.inspect(orderItems));
        logger.info(util.inspect(customerDetails));
        logger.info(util.inspect(customerDetails));
        logger.info(`Order number: ${orderNumber} | method: ${niceShipMethod}`);
        throw new exceptionalError(`Shipment validation failed:\n  -${errMsgs.join("\n  -")}`);
    }
}
// Validate args before looking up a shipment
export function validateShipmentRequest(selectionFilters) {
    const isId = selectionFilters.hasOwnProperty("rslReference");
    const isOrderNumber = selectionFilters.hasOwnProperty("orderNumber");
    let exception = null;
    if (isId && isOrderNumber) {
        logger.info(util.inspect(selectionFilters));
        throw new exceptionalError("rslReference XOR orderNumber");
    }
    if (!isId && !isOrderNumber) {
        logger.info(util.inspect(selectionFilters));
        throw new exceptionalError("Some filter criteria is required");
    }
    if (isId) {
        if (!selectionFilters.rslReference) {
            logger.info(util.inspect(selectionFilters));
            throw new exceptionalError("RSL ID must be non-empty");
        }
        if (!String(selectionFilters.rslReference).match(/^\d+?$/i)) {
            logger.info(util.inspect(selectionFilters));
            throw new exceptionalError("RSL IDs are numeric");
        }
    }
    else {
        if (!selectionFilters.orderNumber || selectionFilters.orderNumber === "") {
            logger.info(util.inspect(selectionFilters));
            throw new exceptionalError("Order number must be non-empty");
        }
    }
}
