export declare const apiConstArgs: {
    orderType: string;
    pageSize: number;
};
export declare const apiOrderNumberParamName = "referenceNumber";
export declare const apiOrderTypeParamName = "orderType";
export declare const methodPost = "POST";
export declare const methodGet = "GET";
export declare const endpointAuth = "Auth";
export declare const endpointShipments = "Orders";
export declare const retrievalTimeoutMs: number;
export declare const tokenLifespan: number;
export declare const refreshTokenThreshold: number;
export interface authBody {
    clientId: string;
    apiUserIdentifier: string;
    apiUserSecret: string;
}
export interface statusMsg {
    statusCode: string;
    statusMessage: string;
}
export interface tokenResp {
    data: {
        token: string;
    };
    status: {
        statusMessage: string;
        statusCodes: statusMsg[];
    };
}
export interface fetchOpts {
    headers?: {
        'Content-Type'?: string;
        Authorization?: string;
        ClientID?: string;
    };
    body?: string;
    method?: string;
}
export declare const rslOrderStatusAccepted = "Accepted";
export declare const rslOrderStatusFulfilling = "InFulfllment";
export declare const rslOrderStatusFulfillingCorrected = "InFulfillment";
export declare const rslOrderStatusShipped = "Shipped";
export declare const rslOrderStatusBackordered = "Backorder";
export declare const rslOrderStatuses: readonly ["Accepted", "InFulfllment", "InFulfillment", "Shipped", "Backorder"];
export declare const shipMethodExpress = "Priority Mail Small Flat Rate Box";
export declare const shipMethodPriority = "2nd Day Air";
export declare const shipMethodStandard = "First Class";
export declare const shipMethodStandardName = "Standard";
export declare const shipMethodExpressName = "Express";
export declare const shipMethodPriorityName = "Priority";
export declare const shipMethodGroundName = "Ground";
export declare const standardShippingMethods: readonly ["Standard", "Express", "Priority", "Ground"];
interface shipMethods {
    [key: string]: shipMethodDetails;
}
export interface shipMethodDetails {
    carrier: string;
    createPermitted: boolean;
    serviceType: typeof standardShippingMethods[number] | null;
}
export declare enum smrtSKUs {
    lsb4 = "BD2",
    accutradeInsert = "AT-Insert",
    compBD = "B07GSFWM3J",
    socket = "Socket",
    lsb2 = "2746-B00652G4TS"
}
export declare const shippingMethods: shipMethods;
declare const listOfShippingMethods: string[];
export type shippingMethodTypes = typeof listOfShippingMethods[number];
export interface rslGetOrderJSON {
    pageMetadata: rslFieldpageMetadata;
    data: rslExtendedOrderFields;
    status: rslFieldStatus;
}
export interface rslGetOrderSJSON {
    pageMetadata: rslFieldpageMetadata;
    data: rslCommonOrderFields[];
    status: rslFieldStatus;
}
interface rslFieldpageMetadata {
    totalResults: number;
    totalPages: number;
    pageSize: number;
    page: number;
}
interface rslFieldStatus {
    statusMessage: string | null;
    statusCodes: [
        {
            statusCode: number;
            statusMessage: string | null;
        }
    ];
}
export interface rslCommonOrderFields {
    orderId: number;
    referenceNumber: string;
    customerEmail: string | null;
    customerPhoneNumber: string | null;
    orderCustomerName: string;
    stockKeepingUnits: rslRequestedSKUs[];
    orderComments: string | null;
    orderType: string;
    statusDescription: typeof rslOrderStatuses[number];
    warehouseName: string | null;
    customerCompany: string | null;
    customerAddress1: string;
    customerAddress2: string | null;
    customerAddress3: string | null;
    customerCity: string;
    customerStateOrProvince: string;
    customerCountry: string;
    customerPostalCode: string;
    shippingMethod: shippingMethodTypes;
    dateCreated: string;
    dateBackordered: string | null;
}
export interface rslExtendedOrderFields extends rslCommonOrderFields {
    carrier: string | null;
    actualShippingMethod: shippingMethodTypes;
    multiCarrierPackage: false;
    packages: rslPackage[];
}
interface rslRequestedSKUs {
    stockKeepingUnit: string;
    description: string;
    quantity: number;
    serialNumbers: [];
    itemLots: [];
}
export interface rslPackage {
    packageId: string;
    dimensions: string | null;
    packageItemCounts: number;
    packageWeight: number;
    shippingDate: string | null;
    carrier: string | null;
    actualShippingMethod: shippingMethodTypes;
    trackingNumber: string | null;
    trackingNumberURL: string | null;
    stockKeepingUnits: [
        {
            stockKeepingUnit: string;
            quantity: number;
        }
    ];
}
export interface rslCreateShipmentJSON {
    referenceNumber: string;
    orderType: "Consumer";
    customerCompanyName?: string;
    customerFullName: string;
    customerStreetAddress1: string;
    customerStreetAddress2?: string;
    customerStreetAddress3?: string;
    customerCity: string;
    customerStateOrProvince: string;
    customerPostalCode: string;
    customerCountry: string;
    customerEmailAddress?: string;
    customerTelephoneNumber?: string;
    orderShippingMethod: string;
    orderComments?: string;
    orderSubTotalAmount?: number;
    orderTaxAmount?: number;
    orderShippingCostAmount?: number;
    orderTotalAmount?: number;
    stockKeepingUnits: newShipmentSku[];
    tradingPartnerId?: string;
    vendorNumber?: string;
    thirdPartyShipAccountNumber?: string;
    warehouseId?: string;
}
export interface newShipmentSku {
    stockKeepingUnit: string;
    quantityOrdered: number;
    unitPrice?: number;
}
export interface shipmentLookupFilters {
    orderNumber?: string;
    rslReference?: string | number;
}
export type niceShipments = niceShipment[];
export interface niceShipment {
    orderNumber: string;
    rslReference: string;
    status: typeof rslOrderStatuses[number];
    receiver?: niceRecipientDetails;
    dateCreated?: Date;
    notes?: string | null;
    reqItems?: niceRequestedItem[];
    requestedMethod?: string;
    shipments: niceShipmentDetails[];
}
export interface niceShipmentDetails {
    shipmentId?: string;
    dimensions?: string | null;
    packageQty?: number;
    weight?: number | null;
    shipDate?: Date | null;
    carrier: string | null;
    shipService: string | null;
    niceService?: typeof standardShippingMethods[number];
    tracking: string | null;
    trackURL?: string | null;
    containedSkus?: {
        [key: string]: number;
    };
}
export interface niceNewOrderSkuMapping {
    [key: string]: {
        qty: number;
        unitPrice?: number;
    };
}
export interface niceRecipientDetails {
    name: string;
    phone?: string | null;
    email?: string | null;
    addressOne: string;
    addressTwo?: string | null;
    addressThree?: string | null;
    company?: string | null;
    city: string;
    state: string;
    country: string;
    zip: string;
}
export interface niceRequestedItem {
    sku: string;
    qty: number;
    description: string | null;
}
export {};
