var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as logger from 'logger';
export { logger };
import * as rslConstants from './smrtfill.constants.js';
import * as rslAPI from './smrtfillAPI.js';
import * as rslUtilities from './smart.utilities.js';
import exceptionalError from 'exceptionalhandler';
export { exceptionalError };
import util from 'util';
// ░██████╗░███████╗████████╗
// ██╔════╝░██╔════╝╚══██╔══╝
// ██║░░██╗░█████╗░░░░░██║░░░
// ██║░░╚██╗██╔══╝░░░░░██║░░░
// ╚██████╔╝███████╗░░░██║░░░
// ░╚═════╝░╚══════╝░░░╚═╝░░░
export function getShipment(selectionFilters) {
    return __awaiter(this, void 0, void 0, function* () {
        rslUtilities.validateShipmentRequest(selectionFilters);
        let endPoint = '';
        let body = {};
        const isRequestByRslId = !!selectionFilters.rslReference;
        let shipmentDetails;
        if (isRequestByRslId) {
            shipmentDetails = yield getShipmentById(selectionFilters);
        }
        else {
            shipmentDetails = yield getShipmentsByFilters(selectionFilters);
        }
        return convertRSLResponseToSomethingNicer(shipmentDetails);
    });
}
function getShipmentsByFilters(selectionFilters) {
    return __awaiter(this, void 0, void 0, function* () {
        const endPoint = `${rslConstants.endpointShipments}?${rslConstants.apiOrderTypeParamName}=${rslConstants.apiConstArgs.orderType}&${rslConstants.apiOrderNumberParamName}=${selectionFilters.orderNumber}`;
        let getShipmentsResp = yield rslAPI.performApiCall(endPoint, rslConstants.methodGet);
        if (getShipmentsResp.pageMetadata.totalResults !== 1 || getShipmentsResp.data.length !== 1) {
            logger.info(util.inspect(selectionFilters));
            logger.info(util.inspect(getShipmentsResp.pageMetadata));
            logger.info(util.inspect(getShipmentsResp.data));
            logger.info(util.inspect(getShipmentsResp.status));
            throw new exceptionalError(`Got ${getShipmentsResp.pageMetadata.totalResults} results for ${selectionFilters.orderNumber}`);
        }
        return yield getShipmentById({ rslReference: String(getShipmentsResp.data[0].orderId) });
    });
}
function getShipmentById(selectionFilters) {
    return __awaiter(this, void 0, void 0, function* () {
        const endPoint = `${rslConstants.endpointShipments}/${selectionFilters.rslReference}`;
        const notFoundErrMsg = `No match for RSL ID ${selectionFilters.rslReference}`;
        let getShipmentResp;
        try {
            getShipmentResp = yield rslAPI.performApiCall(endPoint, rslConstants.methodGet);
        }
        catch (e) {
            // todo - do better once real errors are here
            if (e.message.match(/404|not valid/i)) {
                throw new exceptionalError(notFoundErrMsg);
            }
            else {
                throw e;
            }
        }
        if (Array.isArray(getShipmentResp.data)) {
            if (getShipmentResp.data.length === 0) {
                logger.info(util.inspect(selectionFilters));
                logger.info(util.inspect(getShipmentResp.data));
                throw new exceptionalError(notFoundErrMsg);
            }
            else {
                logger.info(util.inspect(getShipmentResp.data));
                //@ts-ignore
                throw new exceptionalError(`getshipment for RSL ${selectionFilters.rslReference} had ${getShipmentResp.data.length} entries - why am I even checking for this?!?!!?`);
            }
        }
        return getShipmentResp.data;
    });
}
function convertRSLResponseToSomethingNicer(rslShipmentResponse) {
    let totalReqItems = 0;
    let totalSentItems = 0;
    let niceResponse = {
        orderNumber: rslShipmentResponse.referenceNumber,
        rslReference: String(rslShipmentResponse.orderId),
        status: rslShipmentResponse.statusDescription,
        notes: rslShipmentResponse.orderComments,
        requestedMethod: rslShipmentResponse.shippingMethod,
        dateCreated: new Date(rslShipmentResponse.dateCreated),
        receiver: {
            name: rslShipmentResponse.orderCustomerName,
            phone: rslShipmentResponse.customerPhoneNumber,
            email: rslShipmentResponse.customerEmail,
            addressOne: rslShipmentResponse.customerAddress1,
            addressTwo: rslShipmentResponse.customerAddress2,
            addressThree: rslShipmentResponse.customerAddress3,
            company: rslShipmentResponse.customerCompany,
            city: rslShipmentResponse.customerCity,
            state: rslShipmentResponse.customerStateOrProvince,
            country: rslShipmentResponse.customerCountry,
            zip: rslShipmentResponse.customerPostalCode
        },
        reqItems: [],
        shipments: []
    };
    checkShipmentStatus(niceResponse, rslShipmentResponse);
    processShipmentRequestedItems(niceResponse, rslShipmentResponse);
    totalReqItems = niceResponse.reqItems.map((itemRow) => (itemRow.qty)).reduce((reqCount, a) => (reqCount + a), 0);
    if (niceResponse.status === rslConstants.rslOrderStatusShipped) {
        processShipmentPackages(rslShipmentResponse, niceResponse);
        //@ts-ignore
        for (const shipment of niceResponse.shipments) {
            //@ts-ignore
            for (const [sku, qty] of Object.entries(shipment.containedSkus)) {
                totalSentItems += qty;
            }
        }
        if (totalReqItems !== totalSentItems) {
            logger.info(util.inspect(niceResponse.shipments));
            logger.info(util.inspect(niceResponse.reqItems));
            throw new exceptionalError(`Order ${rslShipmentResponse.orderId} had ${totalReqItems} in the request, but ${totalSentItems} were shipped`);
        }
    }
    return niceResponse;
}
function checkShipmentStatus(niceResponse, rslShipmentResponse) {
    if (niceResponse.status === rslConstants.rslOrderStatusBackordered) {
        throw new exceptionalError(`Order ${rslShipmentResponse.orderId} has backordered items which aren't supported right now`);
    }
    if (niceResponse.status === rslConstants.rslOrderStatusFulfilling) {
        niceResponse.status = rslConstants.rslOrderStatusFulfillingCorrected;
    }
}
function processShipmentRequestedItems(niceResponse, rslShipmentResponse) {
    for (const orderLineItem of rslShipmentResponse.stockKeepingUnits) {
        let newOrderLineItem = {
            sku: orderLineItem.stockKeepingUnit,
            qty: orderLineItem.quantity,
            description: orderLineItem.description
        };
        niceResponse.reqItems.push(newOrderLineItem);
    }
}
function processShipmentPackages(rslShipmentResponse, niceResponse) {
    if (rslShipmentResponse.packages.length === 0) {
        logger.info(util.inspect(rslShipmentResponse));
        throw new exceptionalError(`Order ${rslShipmentResponse.orderId} has status ${niceResponse.status} but returned 0 packages`);
    }
    for (const shipmentLineItem of rslShipmentResponse.packages) {
        let totalPackageQty = 0;
        const standardShipServiceDetails = validatePackage(shipmentLineItem, rslShipmentResponse);
        const niceService = standardShipServiceDetails.serviceType;
        if (!niceService) {
            logger.info(util.inspect(rslShipmentResponse));
            logger.info(util.inspect(shipmentLineItem));
            throw new exceptionalError(`Order ${rslShipmentResponse.orderId} with shipment requested type \"${rslShipmentResponse.shippingMethod}\" has unknown method \"${shipmentLineItem.actualShippingMethod}\"`);
        }
        let newShipment = {
            shipmentId: shipmentLineItem.packageId,
            dimensions: shipmentLineItem.dimensions,
            packageQty: shipmentLineItem.packageItemCounts,
            weight: shipmentLineItem.packageWeight,
            shipDate: new Date(shipmentLineItem.shippingDate),
            carrier: shipmentLineItem.carrier,
            shipService: shipmentLineItem.actualShippingMethod,
            tracking: shipmentLineItem.trackingNumber,
            trackURL: shipmentLineItem.trackingNumberURL,
            niceService: niceService,
            containedSkus: {}
        };
        for (const packageLine of shipmentLineItem.stockKeepingUnits) {
            totalPackageQty += packageLine.quantity;
            //@ts-ignore
            newShipment.containedSkus[packageLine.stockKeepingUnit] = packageLine.quantity;
        }
        // no es beuno - packageItemCounts is just the number of packages
        //if (totalPackageQty !== newShipment.itemQty){
        //    logger.info(util.inspect(shipmentLineItem))
        //    throw new exceptionalError(`Order ${rslShipmentResponse.orderId} package ${shipmentLineItem.packageId} had ${shipmentLineItem.packageItemCounts} items but I found ${totalPackageQty}`)
        //}
        niceResponse.shipments.push(newShipment);
    }
}
function validatePackage(shipmentLineItem, rslShipmentResponse) {
    if (!rslConstants.shippingMethods.hasOwnProperty(shipmentLineItem.actualShippingMethod)) {
        logger.info(util.inspect(shipmentLineItem));
        throw new exceptionalError(`Order ${rslShipmentResponse.orderId} has unknown shipping service ${shipmentLineItem.actualShippingMethod}`);
    }
    if (!shipmentLineItem.trackingNumber || !shipmentLineItem.carrier) {
        logger.info(util.inspect(shipmentLineItem));
        throw new exceptionalError(`Order ${rslShipmentResponse.orderId} package ${shipmentLineItem.packageId} had incomplete shipping deets: tracking \"${shipmentLineItem.trackingNumber}\" | carrier ${shipmentLineItem.carrier}`);
    }
    if (rslShipmentResponse.carrier === "XParcel") {
        logger.info(util.inspect(shipmentLineItem));
        throw new exceptionalError(`Order ${rslShipmentResponse.orderId} has carrier XParcel which is not currently supported by my tool`);
    }
    const standardShipServiceDetails = rslConstants.shippingMethods[shipmentLineItem.actualShippingMethod];
    if (rslShipmentResponse.carrier !== standardShipServiceDetails.carrier) {
        logger.info(util.inspect(shipmentLineItem));
        throw new exceptionalError(`Order ${rslShipmentResponse.orderId} has carrier ${shipmentLineItem.carrier} for service ${shipmentLineItem.actualShippingMethod} when it should be ${standardShipServiceDetails.carrier}`);
    }
    return standardShipServiceDetails;
}
// ░█████╗░██████╗░███████╗░█████╗░████████╗███████╗
// ██╔══██╗██╔══██╗██╔════╝██╔══██╗╚══██╔══╝██╔════╝
// ██║░░╚═╝██████╔╝█████╗░░███████║░░░██║░░░█████╗░░
// ██║░░██╗██╔══██╗██╔══╝░░██╔══██║░░░██║░░░██╔══╝░░
// ╚█████╔╝██║░░██║███████╗██║░░██║░░░██║░░░███████╗
// ░╚════╝░╚═╝░░╚═╝╚══════╝╚═╝░░╚═╝░░░╚═╝░░░╚══════╝
export function createShipment(orderNumber, orderItems, customerDetails, niceShipMethod, orderComments = "") {
    return __awaiter(this, void 0, void 0, function* () {
        rslUtilities.validateNewShipment(orderNumber, orderItems, customerDetails, niceShipMethod);
        let newOrderDeets = {
            referenceNumber: orderNumber,
            orderType: rslConstants.apiConstArgs.orderType,
            customerFullName: customerDetails.name,
            customerStreetAddress1: customerDetails.addressOne,
            customerCity: customerDetails.city,
            customerStateOrProvince: customerDetails.state,
            customerPostalCode: customerDetails.zip,
            customerCountry: customerDetails.country,
            orderShippingMethod: '',
            stockKeepingUnits: [],
        };
        if (customerDetails.company) {
            newOrderDeets.customerCompanyName = customerDetails.company;
        }
        addOptionalFields(newOrderDeets, customerDetails, orderComments);
        addSKUs(orderItems, newOrderDeets);
        addShippingMethod(niceShipMethod, newOrderDeets);
        yield rslAPI.performApiCall(rslConstants.endpointShipments, rslConstants.methodPost, newOrderDeets);
        return yield checkCreatedOrder(orderNumber);
    });
}
function addOptionalFields(newOrderDeets, customerDetails, orderComments) {
    const optionalFields = {
        customerEmailAddress: "email",
        customerTelephoneNumber: "phone",
        customerStreetAddress2: "addressTwo",
        customerStreetAddress3: "addressThree"
    };
    for (const [rslName, niceName] of Object.entries(optionalFields)) {
        //@ts-ignore
        if (!rslUtilities.valIsEmpty(customerDetails[niceName])) {
            //@ts-ignore
            newOrderDeets[rslName] = customerDetails[niceName];
        }
    }
    if (!rslUtilities.valIsEmpty(orderComments)) {
        newOrderDeets.orderComments = orderComments;
    }
}
function addSKUs(orderItems, newOrderDeets) {
    for (const [sku, skuDeets] of Object.entries(orderItems)) {
        let unitCost = skuDeets.unitPrice;
        if (!unitCost) {
            unitCost = 0;
        }
        let newSKU = {
            stockKeepingUnit: sku,
            quantityOrdered: skuDeets.qty,
            unitPrice: unitCost
        };
        newOrderDeets.stockKeepingUnits.push(newSKU);
    }
}
function addShippingMethod(niceShipMethod, newOrderDeets) {
    switch (niceShipMethod) {
        case rslConstants.shipMethodStandardName:
            newOrderDeets.orderShippingMethod = rslConstants.shipMethodStandard;
            break;
        case rslConstants.shipMethodExpressName:
            newOrderDeets.orderShippingMethod = rslConstants.shipMethodExpress;
            break;
        case rslConstants.shipMethodPriorityName:
            newOrderDeets.orderShippingMethod = rslConstants.shipMethodPriority;
            break;
    }
}
function checkCreatedOrder(orderNumber) {
    return __awaiter(this, void 0, void 0, function* () {
        const endPoint = `${rslConstants.endpointShipments}?${rslConstants.apiOrderTypeParamName}=${rslConstants.apiConstArgs.orderType}&${rslConstants.apiOrderNumberParamName}=${orderNumber}`;
        const giveUp = Number(new Date()) + rslConstants.retrievalTimeoutMs;
        let found = false;
        //@ts-ignore
        let verification = {};
        while (giveUp - Number(new Date()) > 0) {
            try {
                verification = yield rslAPI.performApiCall(endPoint, rslConstants.methodGet);
            }
            catch (e) {
                if (!e.message.match(/400|404|valid/)) {
                    throw e;
                }
            }
            if (verification.data && verification.data.length === 1) {
                return String(verification.data[0].orderId);
            }
        }
        throw new exceptionalError(`Failed to validate new shipment for order ${orderNumber}`);
    });
}
