var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import chai from 'chai';
import chaiAsPromised from "chai-as-promised";
import fetchMock from 'fetch-mock';
const expect = chai.expect;
chai.use(chaiAsPromised);
import * as rslAPI from '../smrtfillAPI.js';
import * as authCases from './testDetails/smrt.cases.auth.js';
describe('Tests authorization', () => __awaiter(void 0, void 0, void 0, function* () {
    context('Handling failures', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.post(/auth/i, (url, opts) => __awaiter(void 0, void 0, void 0, function* () {
                return authCases.checkAuthReq(url, opts, 0);
            }));
        }));
        it('Catches auth failure exceptions', () => __awaiter(void 0, void 0, void 0, function* () {
            yield expect(rslAPI.authenticate()).to.be.rejectedWith("API credential does not exist");
        }));
    }));
    context('Authorizes successfully', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.post(/auth/i, (url, opts) => __awaiter(void 0, void 0, void 0, function* () {
                return authCases.checkAuthReq(url, opts, 1);
            }));
        }));
        it('Authorizes', () => __awaiter(void 0, void 0, void 0, function* () {
            expect(rslAPI.connected).to.be.false;
            yield expect(rslAPI.authenticate()).not.to.be.rejected;
            expect(rslAPI.connected).to.be.true;
            expect(rslAPI.token).to.equal(authCases.tokenOne);
        }));
    }));
    context('Reauthorizes successfully', () => __awaiter(void 0, void 0, void 0, function* () {
        before(() => __awaiter(void 0, void 0, void 0, function* () {
            fetchMock.restore();
            fetchMock.post(/auth/i, (url, opts) => __awaiter(void 0, void 0, void 0, function* () {
                return authCases.checkAuthReq(url, opts, 2);
            }));
            fetchMock.get(/lol/, {});
        }));
        it('Reauthorizes', () => __awaiter(void 0, void 0, void 0, function* () {
            expect(rslAPI.connected).to.be.true;
            expect(rslAPI.token).to.equal(authCases.tokenOne);
            rslAPI.overrideExpTime(Number((new Date())) + 100);
            yield expect(rslAPI.rawRequest("lol", "GET")).not.to.be.rejected;
            expect(rslAPI.connected).to.be.true;
            expect(rslAPI.token).to.equal(authCases.tokenTwo);
        }));
    }));
    context('Gets paginated results', () => __awaiter(void 0, void 0, void 0, function* () {
        it.skip('Deferred for now since I dont see when Ill be getting >1 result unless something goes terribly wrong?', () => __awaiter(void 0, void 0, void 0, function* () { }));
    }));
}));
