var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import chai from 'chai';
import chaiAsPromised from "chai-as-promised";
import fetchMock from 'fetch-mock';
const expect = chai.expect;
chai.use(chaiAsPromised);
import * as smrt from '../smrtfill.js';
import * as authCases from './testDetails/smrt.cases.auth.js';
import * as getTestCases from './testDetails/smrt.cases.get.js';
describe('I AM SO SMRT', () => __awaiter(void 0, void 0, void 0, function* () {
    before(() => __awaiter(void 0, void 0, void 0, function* () {
        fetchMock.restore();
        fetchMock.post(/auth/i, (url, opts) => __awaiter(void 0, void 0, void 0, function* () {
            return authCases.checkAuthReq(url, opts, 1);
        }));
        fetchMock.get(/./, (url, opts) => __awaiter(void 0, void 0, void 0, function* () {
            return getTestCases.getReq(url, opts);
        }));
        fetchMock.post(/orders/i, (url, opts) => __awaiter(void 0, void 0, void 0, function* () {
            return getTestCases.checkPostBody(url, opts);
        }));
    }));
    context('Gets orders by RSL id', () => __awaiter(void 0, void 0, void 0, function* () {
        it('Validates', () => __awaiter(void 0, void 0, void 0, function* () {
            yield expect(smrt.getShipment({ rslReference: '13123fasfasf' })).to.be.rejectedWith('RSL IDs are numeric');
            yield expect(smrt.getShipment({ rslReference: '13123', orderNumber: "asdsadassd" })).to.be.rejectedWith('rslReference XOR orderNumber');
            //@ts-ignore
            yield expect(smrt.getShipment({ rslReference: null })).to.be.rejectedWith('RSL ID must be non-empty');
        }));
        it('Handles the no-match 400 case', () => __awaiter(void 0, void 0, void 0, function* () {
            const idErr = '400';
            yield expect(smrt.getShipment({ rslReference: idErr })).to.be.rejectedWith('No match for RSL ID 400');
        }));
        it('Handles the no-match 404 case', () => __awaiter(void 0, void 0, void 0, function* () {
            const idErr = '404';
            yield expect(smrt.getShipment({ rslReference: idErr })).to.be.rejectedWith('No match for RSL ID 404');
        }));
        it('Gets orders by RSL ID', () => __awaiter(void 0, void 0, void 0, function* () {
            const idAccepted = '67298';
            let acceptedOrder = yield smrt.getShipment({ rslReference: idAccepted });
            expect(acceptedOrder.orderNumber).to.equal("NEWBDORDER123");
            expect(acceptedOrder.rslReference).to.equal("123456789");
            expect(acceptedOrder.requestedMethod).to.equal("2nd Day Air");
            expect(acceptedOrder.status).to.equal("Accepted");
            expect(acceptedOrder.shipments).to.be.empty;
            expect(acceptedOrder.receiver).to.deep.equal({
                name: "Chris Nevin",
                phone: "",
                email: "cnevin@yert.ca",
                addressOne: "56 Brodie St",
                addressTwo: null,
                addressThree: null,
                company: null,
                city: "Mount Pearl",
                state: "CA",
                country: "US",
                zip: "54321"
            });
            expect(acceptedOrder.reqItems).to.deep.equal([
                {
                    sku: 'BD2',
                    qty: 5,
                    description: "BlueDriver2 Bluetooth Pro OBDII Scan Tool"
                },
                {
                    sku: '2746-B00652G4TS',
                    qty: 2,
                    description: "BlueDriver"
                }
            ]);
            const idFulfilling = '8963217';
            let fulfilledOrder = yield smrt.getShipment({ rslReference: idFulfilling });
            expect(fulfilledOrder.status).to.equal("InFulfillment");
            expect(String(fulfilledOrder.dateCreated)).to.equal(String(new Date("2023-07-17T04:00:18Z")));
        }));
        it('Catches shipment qty mismatches', () => __awaiter(void 0, void 0, void 0, function* () {
            // 6789321
            yield expect(smrt.getShipment({ rslReference: 6789321 })).to.be.rejectedWith('Order 6789321 had 5 in the request, but 3 were shipped');
        }));
        it('Catches shipment carrier detail mismatches', () => __awaiter(void 0, void 0, void 0, function* () {
            // 999888111
            yield expect(smrt.getShipment({ rslReference: 999888111 })).to.be.rejectedWith('Order 999888111 has carrier USPS for service 2nd Day Air when it should be UPS');
        }));
        it('Catches incomplete tracking info', () => __awaiter(void 0, void 0, void 0, function* () {
            // 9876541
            yield expect(smrt.getShipment({ rslReference: "9876541" })).to.be.rejectedWith('Order 9876541 package 536831692 had incomplete shipping deets');
        }));
    }));
    context('Gets orders by our order #', () => __awaiter(void 0, void 0, void 0, function* () {
        it('Validates', () => __awaiter(void 0, void 0, void 0, function* () {
            yield expect(smrt.getShipment({ orderNumber: '' })).to.be.rejectedWith('Order number must be non-empty');
            //@ts-ignore
            yield expect(smrt.getShipment({ orderNumber: null })).to.be.rejectedWith('Order number must be non-empty');
            yield expect(smrt.getShipment({})).to.be.rejectedWith('Some filter criteria is required');
        }));
        it('Handles the no-match case', () => __awaiter(void 0, void 0, void 0, function* () {
            const orderNumber = 'noOrderNumberMatch';
            yield expect(smrt.getShipment({ orderNumber: orderNumber })).to.be.rejectedWith('Got 0 results for noOrderNumberMatch');
        }));
        it('Gets orders by our order #', () => __awaiter(void 0, void 0, void 0, function* () {
            const orderNumber = 'aValidOrderNumber';
            let order = yield smrt.getShipment({ orderNumber: orderNumber });
            expect(order.shipments).to.have.lengthOf(2);
            expect(order.orderNumber).to.equal(orderNumber);
            expect(order.rslReference).to.equal("51341115");
            expect(order.requestedMethod).to.equal("Priority Mail Small Flat Rate Box");
            expect(order.status).to.equal("Shipped");
            expect(order.shipments).to.deep.equal([
                {
                    shipmentId: "536831692",
                    dimensions: "10x6x1",
                    packageQty: 3,
                    weight: 0.45,
                    shipDate: new Date("2023-07-14T13:38:00Z"),
                    carrier: "USPS",
                    shipService: "Priority Mail",
                    niceService: "Express",
                    tracking: "9405511206209899177740",
                    trackURL: "https://tools.usps.com/go/TrackConfirmAction.action?tRef=fullpage&tLc=1&text28777=&tLabels=9405511206209899177740",
                    containedSkus: {
                        "2746-B00652G4TS": 3
                    }
                },
                {
                    shipmentId: "536831693",
                    dimensions: "10x6x1",
                    packageQty: 2,
                    weight: 0.30,
                    shipDate: new Date("2023-07-14T13:52:00Z"),
                    carrier: "USPS",
                    shipService: "Priority Mail",
                    niceService: "Express",
                    tracking: "9405511206209899177741",
                    trackURL: "https://tools.usps.com/go/TrackConfirmAction.action?tRef=fullpage&tLc=1&text28777=&tLabels=9405511206209899177741",
                    containedSkus: {
                        "2746-B00652G4TS": 2
                    }
                }
            ]);
        }));
        it('Catches >1 match', () => __awaiter(void 0, void 0, void 0, function* () {
            const orderNumber = 'tooManyResponses';
            yield expect(smrt.getShipment({ orderNumber: orderNumber })).to.be.rejectedWith('Got 2 results for tooManyResponses');
        }));
    }));
    context('Creates orders', () => __awaiter(void 0, void 0, void 0, function* () {
        it('Validates', () => __awaiter(void 0, void 0, void 0, function* () {
            //@ts-ignore
            yield expect(smrt.createShipment(null, {}, {}, "")).to.be.rejectedWith("Order Number Is Required");
            //@ts-ignore
            yield expect(smrt.createShipment("", {}, {}, "")).to.be.rejectedWith("Order Number Is Required");
            //@ts-ignore
            yield expect(smrt.createShipment("someOrder", {}, {}, "")).to.be.rejectedWith("Missing mandatory customer address field name");
            //@ts-ignore
            yield expect(smrt.createShipment("someOrder", {}, { name: "1", addressOne: "2", city: "3", state: "4", country: "5", zip: "6" }, "")).to.be.rejectedWith("At least one SKU:QTY entry is required");
            //@ts-ignore
            yield expect(smrt.createShipment("someOrder", { "": null }, { name: "1", addressOne: "2", city: "3", state: "4", country: "5", zip: "6" }, "")).to.be.rejectedWith("Empty or null skus found in shipment item list");
            //@ts-ignore
            yield expect(smrt.createShipment("someOrder", { "asdsad": { qty: "5" } }, { name: "1", addressOne: "2", city: "3", state: "4", country: "5", zip: "6" }, "")).to.be.rejectedWith("Non-numeric qty found for sku");
            //@ts-ignore
            yield expect(smrt.createShipment("someOrder", { "asdsad": { qty: 5 } }, { name: "1", addressOne: "2", city: "3", state: "4", country: "5", zip: "6" }, "")).to.be.rejectedWith("Shipping method is required");
            //@ts-ignore
            yield expect(smrt.createShipment("someOrder", { "asdsad": { qty: 5 } }, { name: "1", addressOne: "2", city: "3", state: "4", country: "5", zip: "6" }, "pigeon")).to.be.rejectedWith("Invalid shipping method selected");
        }));
        it('Creates a shipment with nice shipping name', () => __awaiter(void 0, void 0, void 0, function* () {
            const aNiceNewOrder = {
                orderNumber: "shinyBrandNewOrder!",
                person: {
                    name: "Chris Nevin",
                    addressOne: "56 Brodie Street",
                    addressTwo: "Suite 102",
                    addressThree: "3rd Floor",
                    email: "cnevin@yert.ca",
                    phone: "709-749-1029",
                    company: "LEMOOR",
                    city: "Mount Pearl",
                    state: "CA",
                    country: "US",
                    zip: "12345"
                },
                order: { BD3: { qty: 4 }, iceCream: { qty: 9, unitPrice: 9999 } }
            };
            let newOrderId = yield smrt.createShipment(aNiceNewOrder.orderNumber, aNiceNewOrder.order, aNiceNewOrder.person, "Express", "");
            expect(newOrderId).to.equal("1234567898765432");
        }));
        it('Tries to create a shipment but cant validate it - throws exception', function () {
            return __awaiter(this, void 0, void 0, function* () {
                // this.timeout doesn't like arrow functions!
                this.timeout(6000);
                const orderNoMatch = {
                    orderNumber: "someOrderFailsValidation",
                    person: { name: "1", addressOne: "2", city: "3", state: "4", country: "5", zip: "6" },
                    order: { "BD3": { qty: 25 } }
                };
                yield expect(smrt.createShipment(orderNoMatch.orderNumber, orderNoMatch.order, orderNoMatch.person, "Priority", "")).to.be.rejectedWith("Failed to validate new shipment");
            });
        });
    }));
}));
