import * as authCases from './smrt.cases.auth.js';
import * as rslConstants from '../../smrtfill.constants.js';
import util from 'util';
import * as dotenv from 'dotenv';
dotenv.config();
export function checkPostBody(url, opts) {
    if (!authCases.checkToken(opts.headers, authCases.tokenTwo, process.env.DEV_RSL_PROD_CLIENT_ID)) {
        throw new Error(`Rejected due to header auth params - token "${opts.headers.Authorization}" | clientID: "${opts.headers.ClientID}"`);
    }
    const checkBody = JSON.parse(opts.body);
    switch (checkBody.referenceNumber) {
        case "shinyBrandNewOrder!":
            if (checkBody.customerCity !== "Mount Pearl"
                || checkBody.referenceNumber !== "shinyBrandNewOrder!"
                || checkBody.stockKeepingUnits[0].stockKeepingUnit !== "BD3"
                || checkBody.stockKeepingUnits[1].quantityOrdered !== 9
                || checkBody.orderShippingMethod !== rslConstants.shipMethodExpress
                || checkBody.stockKeepingUnits[0].unitPrice !== 0
                || checkBody.stockKeepingUnits[1].unitPrice !== 9999) {
                console.log(util.inspect(checkBody));
                throw `You blew it somehow`;
            }
            break;
        case "someOrderFailsValidation":
            if (checkBody.orderShippingMethod !== rslConstants.shipMethodPriority || checkBody.stockKeepingUnits[0].unitPrice !== 0) {
                throw `Cooked!`;
            }
            break;
        default:
            throw new Error(`y u call post body w/ ${checkBody.referenceNumber}`);
    }
    return 202;
}
export function getReq(url, opts) {
    if (!authCases.checkToken(opts.headers, authCases.tokenTwo, process.env.DEV_RSL_PROD_CLIENT_ID)) {
        throw new Error(`Rejected due to header auth params - token "${opts.headers.Authorization}" | clientID: "${opts.headers.ClientID}"`);
    }
    let responseCode;
    let responseBody;
    switch (url) {
        // BY ID
        case `https://api.fakersl.com/Orders/400`:
            responseBody = {
                "data": [],
                "status": {
                    "statusMessage": "Error",
                    "statusCodes": [
                        {
                            "statusMessage": "The value '45662655690' is not valid.",
                            "field": "orderId"
                        }
                    ]
                }
            };
            responseCode = 400;
            break;
        case `https://api.fakersl.com/Orders/404`:
            responseBody = {};
            responseCode = 404;
            break;
        case `https://api.fakersl.com/Orders/67298`:
            responseBody = orderByIdAccepted;
            responseCode = 200;
            break;
        case `https://api.fakersl.com/Orders/8963217`:
            responseBody = orderByIdFulfilling;
            responseCode = 200;
            break;
        case `https://api.fakersl.com/Orders/6789321`:
            responseBody = orderByIdShipmentQtyMismatch;
            responseCode = 200;
            break;
        case `https://api.fakersl.com/Orders/999888111`:
            responseBody = orderByIdCarrierDetailMismatch;
            responseCode = 200;
            break;
        case `https://api.fakersl.com/Orders/9876541`:
            responseBody = orderByIdTrackingMismatch;
            responseCode = 200;
            break;
        // BY ORDER NUMBER
        case `https://api.fakersl.com/Orders?${rslConstants.apiOrderTypeParamName}=${rslConstants.apiConstArgs.orderType}&${rslConstants.apiOrderNumberParamName}=noOrderNumberMatch`:
        case `https://api.fakersl.com/Orders?${rslConstants.apiOrderTypeParamName}=${rslConstants.apiConstArgs.orderType}&${rslConstants.apiOrderNumberParamName}=someOrderFailsValidation`:
            responseBody = {
                "pageMetadata": {
                    "totalResults": 0,
                    "totalPages": 0,
                    "pageSize": 10,
                    "page": 1
                },
                "data": [],
                "status": {
                    "statusMessage": "Success",
                    "statusCodes": [
                        {
                            "statusCode": 3000,
                            "statusMessage": "Success"
                        }
                    ]
                }
            };
            responseCode = 200;
            break;
        case `https://api.fakersl.com/Orders?${rslConstants.apiOrderTypeParamName}=${rslConstants.apiConstArgs.orderType}&${rslConstants.apiOrderNumberParamName}=aValidOrderNumber`:
            responseBody = orderByOrderNumShipped;
            responseCode = 200;
            break;
        case `https://api.fakersl.com/Orders/51341115`:
            responseBody = getOrderDetailsForShippedOrderByOrderNumberLongTitle;
            responseCode = 200;
            break;
        case `https://api.fakersl.com/Orders?${rslConstants.apiOrderTypeParamName}=${rslConstants.apiConstArgs.orderType}&${rslConstants.apiOrderNumberParamName}=tooManyResponses`:
            responseBody = orderByOrderNumTwoMatches;
            responseCode = 200;
            break;
        case `https://api.fakersl.com/Orders?${rslConstants.apiOrderTypeParamName}=${rslConstants.apiConstArgs.orderType}&${rslConstants.apiOrderNumberParamName}=shinyBrandNewOrder!`:
            responseBody = newOrderReturnedWithId;
            responseCode = 200;
            break;
        default:
            throw new Error(`Unknown url ${url}`);
    }
    return { status: responseCode, body: responseBody };
}
// ██████╗░██╗██████╗░███████╗░█████╗░████████╗  ██████╗░██╗░░░██╗  ██╗██████╗░
// ██╔══██╗██║██╔══██╗██╔════╝██╔══██╗╚══██╔══╝  ██╔══██╗╚██╗░██╔╝  ██║██╔══██╗
// ██║░░██║██║██████╔╝█████╗░░██║░░╚═╝░░░██║░░░  ██████╦╝░╚████╔╝░  ██║██║░░██║
// ██║░░██║██║██╔══██╗██╔══╝░░██║░░██╗░░░██║░░░  ██╔══██╗░░╚██╔╝░░  ██║██║░░██║
// ██████╔╝██║██║░░██║███████╗╚█████╔╝░░░██║░░░  ██████╦╝░░░██║░░░  ██║██████╔╝
// ╚═════╝░╚═╝╚═╝░░╚═╝╚══════╝░╚════╝░░░░╚═╝░░░  ╚═════╝░░░░╚═╝░░░  ╚═╝╚═════╝░
const orderByIdAccepted = {
    "data": {
        "carrier": "",
        "actualShippingMethod": "",
        "multiCarrierPackage": false,
        "packages": [],
        "orderId": 123456789,
        "referenceNumber": "NEWBDORDER123",
        "customerEmail": "cnevin@yert.ca",
        "customerPhoneNumber": "",
        "orderCustomerName": "Chris Nevin",
        "stockKeepingUnits": [
            {
                "stockKeepingUnit": "BD2",
                "description": "BlueDriver2 Bluetooth Pro OBDII Scan Tool",
                "quantity": 5,
                "serialNumbers": [],
                "itemLots": []
            },
            {
                "stockKeepingUnit": "2746-B00652G4TS",
                "description": "BlueDriver",
                "quantity": 2,
                "serialNumbers": [],
                "itemLots": []
            }
        ],
        "orderComments": "someemail@gmail.com",
        "orderType": "Consumer",
        "statusDescription": "Accepted",
        "warehouseName": "Homer",
        "customerCompany": null,
        "customerAddress1": "56 Brodie St",
        "customerAddress2": null,
        "customerAddress3": null,
        "customerCity": "Mount Pearl",
        "customerStateOrProvince": "CA",
        "customerCountry": "US",
        "customerPostalCode": "54321",
        "shippingMethod": "2nd Day Air",
        "dateCreated": "1985-05-21T00:00:00Z",
        "dateBackordered": null
    },
    "status": {
        "statusMessage": "Success",
        "statusCodes": [
            {
                "statusCode": 3000,
                "statusMessage": "Success"
            }
        ]
    }
};
const orderByIdFulfilling = {
    "data": {
        "carrier": "",
        "actualShippingMethod": "",
        "multiCarrierPackage": false,
        "packages": [],
        "orderId": 51379213,
        "referenceNumber": "BD7291",
        "customerEmail": "fbatracking@obd2blue.com",
        "customerPhoneNumber": "",
        "orderCustomerName": "ASDASDASDSAD",
        "stockKeepingUnits": [
            {
                "stockKeepingUnit": "2746-B00652G4TS",
                "description": "BlueDriver",
                "quantity": 5,
                "serialNumbers": [],
                "itemLots": []
            }
        ],
        "orderComments": "donowong6@gmail.com",
        "orderType": "Consumer",
        "statusDescription": "InFulfillment",
        "warehouseName": "Homer",
        "customerCompany": null,
        "customerAddress1": "15904 87th Road",
        "customerAddress2": null,
        "customerAddress3": null,
        "customerCity": "Jamaica",
        "customerStateOrProvince": "NY",
        "customerCountry": "US",
        "customerPostalCode": "11532",
        "shippingMethod": "Priority Mail Small Flat Rate Box",
        "dateCreated": "2023-07-17T04:00:18Z",
        "dateBackordered": null
    },
    "status": {
        "statusMessage": "Success",
        "statusCodes": [
            {
                "statusCode": 3000,
                "statusMessage": "Success"
            }
        ]
    }
};
// ██████╗░██╗░░░██╗  ░█████╗░██████╗░██████╗░███████╗██████╗░  ███╗░░██╗██╗░░░██╗███╗░░░███╗██████╗░███████╗██████╗░
// ██╔══██╗╚██╗░██╔╝  ██╔══██╗██╔══██╗██╔══██╗██╔════╝██╔══██╗  ████╗░██║██║░░░██║████╗░████║██╔══██╗██╔════╝██╔══██╗
// ██████╦╝░╚████╔╝░  ██║░░██║██████╔╝██║░░██║█████╗░░██████╔╝  ██╔██╗██║██║░░░██║██╔████╔██║██████╦╝█████╗░░██████╔╝
// ██╔══██╗░░╚██╔╝░░  ██║░░██║██╔══██╗██║░░██║██╔══╝░░██╔══██╗  ██║╚████║██║░░░██║██║╚██╔╝██║██╔══██╗██╔══╝░░██╔══██╗
// ██████╦╝░░░██║░░░  ╚█████╔╝██║░░██║██████╔╝███████╗██║░░██║  ██║░╚███║╚██████╔╝██║░╚═╝░██║██████╦╝███████╗██║░░██║
// ╚═════╝░░░░╚═╝░░░  ░╚════╝░╚═╝░░╚═╝╚═════╝░╚══════╝╚═╝░░╚═╝  ╚═╝░░╚══╝░╚═════╝░╚═╝░░░░░╚═╝╚═════╝░╚══════╝╚═╝░░╚═╝
const orderByOrderNumShipped = {
    "pageMetadata": {
        "totalResults": 1,
        "totalPages": 1,
        "pageSize": 10,
        "page": 1
    },
    "data": [
        {
            "orderId": 51341115,
            "referenceNumber": "aValidOrderNumber",
            "customerEmail": "fbatracking@obd2blue.com",
            "customerPhoneNumber": "",
            "orderCustomerName": "First Lastnameman",
            "stockKeepingUnits": [
                {
                    "stockKeepingUnit": "2746-B00652G4TS",
                    "description": "BlueDriver",
                    "quantity": 1,
                    "serialNumbers": [],
                    "itemLots": []
                }
            ],
            "orderComments": "whartman@some@dude.com.com",
            "orderType": "Consumer",
            "statusDescription": "Shipped",
            "warehouseName": "McCook",
            "customerCompany": null,
            "customerAddress1": "65498A willow view",
            "customerAddress2": null,
            "customerAddress3": null,
            "customerCity": "BRISTOL",
            "customerStateOrProvince": "IN",
            "customerCountry": "US",
            "customerPostalCode": "46705",
            "shippingMethod": "Priority Mail Small Flat Rate Box",
            "dateCreated": "2023-07-13T22:00:22Z",
            "dateBackordered": null
        }
    ],
    "status": {
        "statusMessage": "Success",
        "statusCodes": [
            {
                "statusCode": 3000,
                "statusMessage": "Success"
            }
        ]
    }
};
const getOrderDetailsForShippedOrderByOrderNumberLongTitle = {
    "data": {
        "carrier": "USPS",
        "actualShippingMethod": "Priority Mail",
        "multiCarrierPackage": false,
        "packages": [
            {
                "packageId": "536831692",
                "dimensions": "10x6x1",
                "packageItemCounts": 3,
                "packageWeight": 0.45,
                "shippingDate": "2023-07-14T13:38:00Z",
                "carrier": "USPS",
                "actualShippingMethod": "Priority Mail",
                "trackingNumber": "9405511206209899177740",
                "trackingNumberURL": "https://tools.usps.com/go/TrackConfirmAction.action?tRef=fullpage&tLc=1&text28777=&tLabels=9405511206209899177740",
                "stockKeepingUnits": [
                    {
                        "stockKeepingUnit": "2746-B00652G4TS",
                        "quantity": 3
                    }
                ]
            },
            {
                "packageId": "536831693",
                "dimensions": "10x6x1",
                "packageItemCounts": 2,
                "packageWeight": 0.30,
                "shippingDate": "2023-07-14T13:52:00Z",
                "carrier": "USPS",
                "actualShippingMethod": "Priority Mail",
                "trackingNumber": "9405511206209899177741",
                "trackingNumberURL": "https://tools.usps.com/go/TrackConfirmAction.action?tRef=fullpage&tLc=1&text28777=&tLabels=9405511206209899177741",
                "stockKeepingUnits": [
                    {
                        "stockKeepingUnit": "2746-B00652G4TS",
                        "quantity": 2
                    }
                ]
            }
        ],
        "orderId": 51341115,
        "referenceNumber": "aValidOrderNumber",
        "customerEmail": "fbatracking@obd2blue.com",
        "customerPhoneNumber": "",
        "orderCustomerName": "First Lastnameman",
        "stockKeepingUnits": [
            {
                "stockKeepingUnit": "2746-B00652G4TS",
                "description": "BlueDriver",
                "quantity": 5,
                "serialNumbers": [],
                "itemLots": []
            }
        ],
        "orderComments": "some@dude.com",
        "orderType": "Consumer",
        "statusDescription": "Shipped",
        "warehouseName": "McCook",
        "customerCompany": null,
        "customerAddress1": "65498A willow view",
        "customerAddress2": null,
        "customerAddress3": null,
        "customerCity": "BRISTOL",
        "customerStateOrProvince": "IN",
        "customerCountry": "US",
        "customerPostalCode": "46705",
        "shippingMethod": "Priority Mail Small Flat Rate Box",
        "dateCreated": "2023-07-13T22:00:22Z",
        "dateBackordered": null
    },
    "status": {
        "statusMessage": "Success",
        "statusCodes": [
            {
                "statusCode": 3000,
                "statusMessage": "Success"
            }
        ]
    }
};
const newOrderReturnedWithId = {
    "pageMetadata": {
        "totalResults": 1,
        "totalPages": 1,
        "pageSize": 10,
        "page": 1
    },
    "data": [
        {
            "orderId": 1234567898765432,
            "referenceNumber": "shinyBrandNewOrder!",
            "customerEmail": "cnevin@yert.ca",
            "customerPhoneNumber": "709-749-1029",
            "orderCustomerName": "Chris Nevin",
            "stockKeepingUnits": [
                {
                    "stockKeepingUnit": "2746-B00652G4TS",
                    "description": "BlueDriver",
                    "quantity": 1,
                    "serialNumbers": [],
                    "itemLots": []
                }
            ],
            "orderComments": "whartman@some@dude.com.com",
            "orderType": "Consumer",
            "statusDescription": "Shipped",
            "warehouseName": "McCook",
            "customerCompany": "LEMOOR",
            "customerAddress1": "65498A willow view",
            "customerAddress2": null,
            "customerAddress3": null,
            "customerCity": "Mount Pearl",
            "customerStateOrProvince": "CA",
            "customerCountry": "US",
            "customerPostalCode": "123456",
            "shippingMethod": "First Class",
            "dateCreated": "2023-07-13T22:00:22Z",
            "dateBackordered": null
        }
    ],
    "status": {
        "statusMessage": "Success",
        "statusCodes": [
            {
                "statusCode": 3000,
                "statusMessage": "Success"
            }
        ]
    }
};
// ███████╗██████╗░██████╗░░█████╗░██████╗░░██████╗
// ██╔════╝██╔══██╗██╔══██╗██╔══██╗██╔══██╗██╔════╝
// █████╗░░██████╔╝██████╔╝██║░░██║██████╔╝╚█████╗░
// ██╔══╝░░██╔══██╗██╔══██╗██║░░██║██╔══██╗░╚═══██╗
// ███████╗██║░░██║██║░░██║╚█████╔╝██║░░██║██████╔╝
// ╚══════╝╚═╝░░╚═╝╚═╝░░╚═╝░╚════╝░╚═╝░░╚═╝╚═════╝░
const orderByOrderNumTwoMatches = {
    "pageMetadata": {
        "totalResults": 2,
        "totalPages": 1,
        "pageSize": 10,
        "page": 1
    },
    "data": [],
    "status": {
        "statusMessage": "Success",
        "statusCodes": [
            {
                "statusCode": 3000,
                "statusMessage": "Success"
            }
        ]
    }
};
const orderByIdShipmentQtyMismatch = {
    "data": {
        "carrier": "USPS",
        "actualShippingMethod": "Priority Mail",
        "multiCarrierPackage": false,
        "packages": [
            {
                "packageId": "536831692",
                "dimensions": "10x6x1",
                "packageItemCounts": 3,
                "packageWeight": 0.45,
                "shippingDate": "2023-07-14T13:38:00Z",
                "carrier": "USPS",
                "actualShippingMethod": "Priority Mail",
                "trackingNumber": "9405511206209899177740",
                "trackingNumberURL": "https://tools.usps.com/go/TrackConfirmAction.action?tRef=fullpage&tLc=1&text28777=&tLabels=9405511206209899177740",
                "stockKeepingUnits": [
                    {
                        "stockKeepingUnit": "2746-B00652G4TS",
                        "quantity": 3
                    }
                ]
            }
        ],
        "orderId": 6789321,
        "referenceNumber": "113-4900157-6465852-2",
        "customerEmail": "fbatracking@obd2blue.com",
        "customerPhoneNumber": "",
        "orderCustomerName": "First Lastnameman",
        "stockKeepingUnits": [
            {
                "stockKeepingUnit": "2746-B00652G4TS",
                "description": "BlueDriver",
                "quantity": 5,
                "serialNumbers": [],
                "itemLots": []
            }
        ],
        "orderComments": "some@dude.com",
        "orderType": "Consumer",
        "statusDescription": "Shipped",
        "warehouseName": "McCook",
        "customerCompany": null,
        "customerAddress1": "65498A willow view",
        "customerAddress2": null,
        "customerAddress3": null,
        "customerCity": "BRISTOL",
        "customerStateOrProvince": "IN",
        "customerCountry": "US",
        "customerPostalCode": "46705",
        "shippingMethod": "Priority Mail Small Flat Rate Box",
        "dateCreated": "2023-07-13T22:00:22Z",
        "dateBackordered": null
    },
    "status": {
        "statusMessage": "Success",
        "statusCodes": [
            {
                "statusCode": 3000,
                "statusMessage": "Success"
            }
        ]
    }
};
const orderByIdCarrierDetailMismatch = {
    "data": {
        "carrier": "USPS",
        "actualShippingMethod": "Priority Mail",
        "multiCarrierPackage": false,
        "packages": [
            {
                "packageId": "536831692",
                "dimensions": "10x6x1",
                "packageItemCounts": 3,
                "packageWeight": 0.45,
                "shippingDate": "2023-07-14T13:38:00Z",
                "carrier": "USPS",
                "actualShippingMethod": "2nd Day Air",
                "trackingNumber": "9405511206209899177740",
                "trackingNumberURL": "https://tools.usps.com/go/TrackConfirmAction.action?tRef=fullpage&tLc=1&text28777=&tLabels=9405511206209899177740",
                "stockKeepingUnits": [
                    {
                        "stockKeepingUnit": "2746-B00652G4TS",
                        "quantity": 3
                    }
                ]
            }
        ],
        "orderId": 999888111,
        "referenceNumber": "113-4900157-6465852-2",
        "customerEmail": "fbatracking@obd2blue.com",
        "customerPhoneNumber": "",
        "orderCustomerName": "First Lastnameman",
        "stockKeepingUnits": [
            {
                "stockKeepingUnit": "2746-B00652G4TS",
                "description": "BlueDriver",
                "quantity": 5,
                "serialNumbers": [],
                "itemLots": []
            }
        ],
        "orderComments": "some@dude.com",
        "orderType": "Consumer",
        "statusDescription": "Shipped",
        "warehouseName": "McCook",
        "customerCompany": null,
        "customerAddress1": "65498A willow view",
        "customerAddress2": null,
        "customerAddress3": null,
        "customerCity": "BRISTOL",
        "customerStateOrProvince": "IN",
        "customerCountry": "US",
        "customerPostalCode": "46705",
        "shippingMethod": "Priority Mail Small Flat Rate Box",
        "dateCreated": "2023-07-13T22:00:22Z",
        "dateBackordered": null
    },
    "status": {
        "statusMessage": "Success",
        "statusCodes": [
            {
                "statusCode": 3000,
                "statusMessage": "Success"
            }
        ]
    }
};
const orderByIdTrackingMismatch = {
    "data": {
        "carrier": "USPS",
        "actualShippingMethod": "Priority Mail",
        "multiCarrierPackage": false,
        "packages": [
            {
                "packageId": "536831692",
                "dimensions": "10x6x1",
                "packageItemCounts": 3,
                "packageWeight": 0.45,
                "shippingDate": "2023-07-14T13:38:00Z",
                "carrier": null,
                "actualShippingMethod": "2nd Day Air",
                "trackingNumber": "9405511206209899177740",
                "trackingNumberURL": "https://tools.usps.com/go/TrackConfirmAction.action?tRef=fullpage&tLc=1&text28777=&tLabels=9405511206209899177740",
                "stockKeepingUnits": [
                    {
                        "stockKeepingUnit": "2746-B00652G4TS",
                        "quantity": 5
                    }
                ]
            }
        ],
        "orderId": 9876541,
        "referenceNumber": "113-4900157-6465852-2",
        "customerEmail": "fbatracking@obd2blue.com",
        "customerPhoneNumber": "",
        "orderCustomerName": "First Lastnameman",
        "stockKeepingUnits": [
            {
                "stockKeepingUnit": "2746-B00652G4TS",
                "description": "BlueDriver",
                "quantity": 5,
                "serialNumbers": [],
                "itemLots": []
            }
        ],
        "orderComments": "some@dude.com",
        "orderType": "Consumer",
        "statusDescription": "Shipped",
        "warehouseName": "McCook",
        "customerCompany": null,
        "customerAddress1": "65498A willow view",
        "customerAddress2": null,
        "customerAddress3": null,
        "customerCity": "BRISTOL",
        "customerStateOrProvince": "IN",
        "customerCountry": "US",
        "customerPostalCode": "46705",
        "shippingMethod": "Priority Mail Small Flat Rate Box",
        "dateCreated": "2023-07-13T22:00:22Z",
        "dateBackordered": null
    },
    "status": {
        "statusMessage": "Success",
        "statusCodes": [
            {
                "statusCode": 3000,
                "statusMessage": "Success"
            }
        ]
    }
};
