// API Stuffs
export const apiConstArgs = {
    orderType: "Consumer",
    pageSize: 10
}
export const apiOrderNumberParamName = 'referenceNumber'
export const apiOrderTypeParamName = 'orderType'
export const methodPost = "POST";
export const methodGet = "GET"
export const endpointAuth = 'Auth'
export const endpointShipments = 'Orders'
export const retrievalTimeoutMs = 15 * 1000;
export const tokenLifespan: number = 5 * 60 * 1000;
export const refreshTokenThreshold = 30 * 1000;

export interface authBody{
    clientId: string,
    apiUserIdentifier: string,
    apiUserSecret: string
}

export interface statusMsg{
    statusCode: string,
    statusMessage: string
}

export interface tokenResp{
    data: {
        token: string
    },
    status: {
        statusMessage: string,
        statusCodes: statusMsg[]
    }
}

export interface fetchOpts{
    headers?: {
        'Content-Type'?: string,
        Authorization?: string,
        ClientID?: string
    },
    body?: string,
    method?: string
}


// Standard RSL Strings & Values
export const rslOrderStatusAccepted = "Accepted"
export const rslOrderStatusFulfilling = "InFulfllment"
export const rslOrderStatusFulfillingCorrected = "InFulfillment";
export const rslOrderStatusShipped = "Shipped"
export const rslOrderStatusBackordered = "Backorder"
export const rslOrderStatuses = [rslOrderStatusAccepted, rslOrderStatusFulfilling, rslOrderStatusFulfillingCorrected, rslOrderStatusShipped, rslOrderStatusBackordered] as const
export const shipMethodExpress = 'Priority Mail Small Flat Rate Box';
export const shipMethodPriority = '2nd Day Air';
export const shipMethodStandard = 'First Class';
export const shipMethodStandardName = "Standard";
export const shipMethodExpressName = "Express";
export const shipMethodPriorityName = "Priority";
export const shipMethodGroundName = "Ground";
export const standardShippingMethods = [shipMethodStandardName, shipMethodExpressName, shipMethodPriorityName, shipMethodGroundName] as const


// Shipping methods as of 18-Jul-23
interface shipMethods{
    [key: string]: shipMethodDetails
}
export interface shipMethodDetails{
    carrier: string,
    createPermitted: boolean,
    serviceType: typeof standardShippingMethods[number]|null
} 

export enum smrtSKUs{
    lsb4 = "BD2",
    accutradeInsert = "AT-Insert",
    compBD = "B07GSFWM3J",
    socket = "Socket",
    lsb2 = "2746-B00652G4TS",
}

export const shippingMethods: shipMethods = {
    "GlobalMail Packet Plus": {carrier: "DHL", createPermitted: false, serviceType:  null},
    "GlobalMail Parcel": {carrier: "DHL", createPermitted: false, serviceType:  null},
    "GlobalMail Parcel Direct": {carrier: "DHL", createPermitted: false, serviceType:  null},
    "Express Worldwide (NonDoc)": {carrier: "DHL", createPermitted: false, serviceType:  null},
    "First Class Mail": {carrier: "USPS", createPermitted: false, serviceType:  shipMethodStandardName},
    "First Class Mail-Signature Required": {carrier: "USPS", createPermitted: false, serviceType:  shipMethodStandardName},
    "First Class Mail International": {carrier: "USPS", createPermitted: false, serviceType:  shipMethodStandardName},
    "Priority Mail": {carrier: "USPS", createPermitted: false, serviceType:  shipMethodExpressName},
    "Priority Mail-Signature Required": {carrier: "USPS", createPermitted: false, serviceType:  shipMethodExpressName},
    "Priority Mail International": {carrier: "USPS", createPermitted: false, serviceType:  shipMethodExpressName},
    "Priority Mail Small Flat Rate Box": {carrier: "USPS", createPermitted: false, serviceType:  shipMethodExpressName},
    "Priority Mail Medium Flat Rate Box": {carrier: "USPS", createPermitted: false, serviceType:  shipMethodExpressName},
    "Priority Mail Large Flat Rate Box": {carrier: "USPS", createPermitted: false, serviceType:  shipMethodExpressName},
    "Priority Mail Flat Rate Envelope": {carrier: "USPS", createPermitted: false, serviceType:  shipMethodExpressName},
    "Express Mail": {carrier: "USPS", createPermitted: false, serviceType:  shipMethodExpressName},
    "Express Mail International": {carrier: "USPS", createPermitted: false, serviceType:  shipMethodExpressName},
    "Express Flat Rate Envelope": {carrier: "USPS", createPermitted: false, serviceType:  shipMethodExpressName},
    "Media Mail": {carrier: "USPS", createPermitted: false, serviceType:  null},
    "Media Mail-Signature Required": {carrier: "USPS", createPermitted: false, serviceType:  null},
    "SurePost": {carrier: "UPS", createPermitted: false, serviceType:  null},
    "Ground": {carrier: "UPS", createPermitted: false, serviceType:  "Ground"},
    "Ground-Signature Required": {carrier: "UPS", createPermitted: false, serviceType:  null},
    "3-Day Select": {carrier: "UPS", createPermitted: false, serviceType:  shipMethodExpressName},
    "3-Day Select-Signature Required": {carrier: "UPS", createPermitted: false, serviceType:  shipMethodExpressName},
    "2nd Day Air": {carrier: "UPS", createPermitted: false, serviceType:  shipMethodPriorityName},
    "2nd Day Air-Signature Required": {carrier: "UPS", createPermitted: false, serviceType:  shipMethodPriorityName},
    "2nd Day Air AM": {carrier: "UPS", createPermitted: false, serviceType:  shipMethodPriorityName},
    "Next Day Air Saver": {carrier: "UPS", createPermitted: false, serviceType:  shipMethodPriorityName},
    "Next Day Air Saver-Signature Required": {carrier: "UPS", createPermitted: false, serviceType:  shipMethodPriorityName},
    "Next Day Air": {carrier: "UPS", createPermitted: false, serviceType:  shipMethodPriorityName},
    "Next Day Air-Signature Required": {carrier: "UPS", createPermitted: false, serviceType:  shipMethodPriorityName},
    "Next Day Air Saturday Delivery": {carrier: "UPS", createPermitted: false, serviceType:  shipMethodPriorityName},
    "Standard": {carrier: "UPS", createPermitted: false, serviceType:  shipMethodStandardName},
    "Worldwide Express": {carrier: "UPS", createPermitted: false, serviceType:  shipMethodExpressName},
    "Worldwide Express Plus": {carrier: "UPS", createPermitted: false, serviceType:  shipMethodExpressName},
    "Worldwide Express Saver": {carrier: "UPS", createPermitted: false, serviceType:  shipMethodExpressName},
    "Worldwide Expedited": {carrier: "UPS", createPermitted: false, serviceType:  shipMethodExpressName}
}
const listOfShippingMethods = Object.keys(shippingMethods);
export type shippingMethodTypes = typeof listOfShippingMethods[number]


// Types & Interfaces: RSL >> US
export interface rslGetOrderJSON{
    pageMetadata: rslFieldpageMetadata,
    data: rslExtendedOrderFields,
    status: rslFieldStatus
}

export interface rslGetOrderSJSON{
    pageMetadata: rslFieldpageMetadata,
    data: rslCommonOrderFields[],
    status: rslFieldStatus
}

interface rslFieldpageMetadata{
    totalResults: number,
    totalPages: number,
    pageSize: number,
    page: number
}

interface rslFieldStatus{
    statusMessage: string|null,
    statusCodes: [{
        statusCode: number
        statusMessage: string|null
    }]
}

export interface rslCommonOrderFields{
    orderId: number,
    referenceNumber: string,
    customerEmail: string|null,
    customerPhoneNumber: string|null,
    orderCustomerName: string,
    stockKeepingUnits: rslRequestedSKUs[],
    orderComments: string|null,
    orderType: string,
    statusDescription: typeof rslOrderStatuses[number],
    warehouseName: string|null,
    customerCompany: string|null,
    customerAddress1: string,
    customerAddress2: string|null,
    customerAddress3: string|null,
    customerCity: string,
    customerStateOrProvince: string,
    customerCountry: string,
    customerPostalCode: string,
    shippingMethod: shippingMethodTypes,
    dateCreated: string,
    dateBackordered: string|null
}

export interface rslExtendedOrderFields extends rslCommonOrderFields{
    carrier: string|null,
    actualShippingMethod: shippingMethodTypes,
    multiCarrierPackage: false,
    packages: rslPackage[]
}

interface rslRequestedSKUs{
    stockKeepingUnit: string,
    description: string,
    quantity: number
    serialNumbers: [],
    itemLots: []
}

export interface rslPackage{
	packageId: string,
	dimensions: string|null,
	packageItemCounts: number,
	packageWeight: number,
	shippingDate: string|null,
	carrier: string|null,
	actualShippingMethod: shippingMethodTypes,
	trackingNumber: string|null,
	trackingNumberURL: string|null,
	stockKeepingUnits: [
		{
			stockKeepingUnit: string
			quantity: number
		}
	]
}

export interface rslCreateShipmentJSON{
    referenceNumber: string,
    orderType: "Consumer",
    customerCompanyName?: string,
    customerFullName: string,
    customerStreetAddress1: string,
    customerStreetAddress2?: string,
    customerStreetAddress3?: string,
    customerCity: string,
    customerStateOrProvince: string,
    customerPostalCode: string,
    customerCountry: string,
    customerEmailAddress?: string,
    customerTelephoneNumber?: string,
    orderShippingMethod: string,
    orderComments?: string,
    orderSubTotalAmount?: number,
    orderTaxAmount?: number,
    orderShippingCostAmount?: number,
    orderTotalAmount?: number,
    stockKeepingUnits: newShipmentSku[],
    tradingPartnerId?: string,
    vendorNumber?: string,
    thirdPartyShipAccountNumber?: string,
    warehouseId?: string
}

export interface newShipmentSku{
    stockKeepingUnit: string,
    quantityOrdered: number,
    unitPrice?: number
}


// Types & Interfaces: US >> RSL
export interface shipmentLookupFilters{
    orderNumber?: string,
    rslReference?: string|number
}

export type niceShipments = niceShipment[];
export interface niceShipment{
    orderNumber: string,
    rslReference: string,
    status: typeof rslOrderStatuses[number],
    receiver?: niceRecipientDetails,
    dateCreated?: Date,
    notes?: string|null, 
    reqItems?: niceRequestedItem[],
    requestedMethod?: string
    shipments: niceShipmentDetails[]
}

export interface niceShipmentDetails{
    shipmentId?: string;
    dimensions?: string | null;
    packageQty?: number;
    weight?: number | null;
    shipDate?: Date | null;
    carrier: string | null;
    shipService: string | null;
    niceService?: typeof standardShippingMethods[number];
    tracking: string | null;
    trackURL?: string | null;
    containedSkus?: {
        [key: string]: number;
    };
}

export interface niceNewOrderSkuMapping{
    [key: string]: {
        qty: number,
        unitPrice?: number
    }
}

export interface niceRecipientDetails{
    name: string,
    phone?: string|null,
    email?: string|null,
    addressOne: string,
    addressTwo?: string|null,
    addressThree?: string|null,
    company?: string|null,
    city: string,
    state: string,
    country: string,
    zip: string
}

export interface niceRequestedItem{
    sku: string,
    qty: number,
    description: string|null
}