import chai from 'chai';
import chaiAsPromised from "chai-as-promised";
import fetch from 'node-fetch';
import fetchMock from 'fetch-mock';
const expect = chai.expect;
chai.use(chaiAsPromised);

import * as rslAPI from '../smrtfillAPI.js';
import * as authCases from './testDetails/smrt.cases.auth.js'

describe('Tests authorization', async() => {   
    context('Handling failures', async() => {

        before(async() => {
            fetchMock.restore();
            fetchMock.post(/auth/i, async (url, opts) => {
                return authCases.checkAuthReq(url, opts, 0)
            });
        });

        it('Catches auth failure exceptions', async() => {
            await expect(rslAPI.authenticate()).to.be.rejectedWith("API credential does not exist")
        })
    })

    context('Authorizes successfully', async() => {
        before(async() => {
            fetchMock.restore();
            fetchMock.post(/auth/i, async (url, opts) => {
                return authCases.checkAuthReq(url, opts, 1)
            });
        });

        it('Authorizes', async() => {
            expect(rslAPI.connected).to.be.false;
            await expect(rslAPI.authenticate()).not.to.be.rejected;
            expect(rslAPI.connected).to.be.true
            expect(rslAPI.token).to.equal(authCases.tokenOne)
        })
    })

    context('Reauthorizes successfully', async() => {
        before(async() => {
            fetchMock.restore();
            fetchMock.post(/auth/i, async (url, opts) => {
                return authCases.checkAuthReq(url, opts, 2)
            });
            fetchMock.get(/lol/, {})
        });

        it('Reauthorizes', async() => {
            expect(rslAPI.connected).to.be.true;
            expect(rslAPI.token).to.equal(authCases.tokenOne)
            rslAPI.overrideExpTime(Number((new Date())) + 100)
            await expect(rslAPI.rawRequest("lol", "GET")).not.to.be.rejected;
            expect(rslAPI.connected).to.be.true
            expect(rslAPI.token).to.equal(authCases.tokenTwo)
        })
    })

    context('Gets paginated results', async() => {
        it.skip('Deferred for now since I dont see when Ill be getting >1 result unless something goes terribly wrong?', async () => {});
    })


})







